# -*- coding: utf-8 -*-


'''
wx.lib.editor.Editor is buggy. This class implements a basic custom text 
editor.
'''

import os, sys
import wx, codecs
from messages import *
import locale

class TextEditor(wx.Frame):
	def __init__(self, parent, title = '', text = '', wildcard = "Text files (*.txt)|*.txt",
				 visible = True, defaultFile = "transpraat.txt"):
		if sys.platform.startswith('win'):
			size = (550, 550)
		else:
			size = (540, 525)
		
		wx.Frame.__init__(self, parent, -1, title, (50, 50), size)
		self.panel = wx.Panel(self, -1)
		self.wildcard = wildcard
		self.defaultFile = defaultFile
		self.title = title
		self.SetEncoding()
		
		self.CreateMenu()
		self.SetPanel(text)
		
		self.Show(visible)
		
	def SetEncoding(self):
		self.encoding = locale.getpreferredencoding()
##		if sys.platform[:3] == 'win':
##			self.encoding = ''
##			
##		elif sys.platform == 'darwin':
##			self.encoding = 'mac-roman'

		
	def SetPanel(self, text):
		self.txt = wx.TextCtrl(self, -1, text, wx.DefaultPosition,
				size=(540, 500), style=wx.TE_MULTILINE)

		
	def CreateMenu(self):
		menuBar = wx.MenuBar()
		
		# file menu
		fileMenu = wx.Menu()
		saveMenuItem = fileMenu.Append(wx.ID_SAVE, '&Save as...\tctrl-s')
		fileMenu.AppendSeparator()			
		quitMenuItem = fileMenu.Append(-1, '&Close\tctrl-w')
	
		self.Bind(wx.EVT_MENU, self.OnSave, saveMenuItem)
		self.Bind(wx.EVT_MENU, self.OnQuit, quitMenuItem)
		
		menuBar.Append(fileMenu, 'File')
		self.SetMenuBar(menuBar)	
		
	
	def OnOpen(self, e):
		info(u'Not implemented')
		
	def OnSave(self, e, content = None):
		path = self.SaveAsFileName()
		if not content:
			end  = self.txt.GetLastPosition()
			content = self.txt.GetString(0, end)
		self.WriteFile(path, content)
		
		
	def OnQuit(self, e):
		self.Destroy()
		
		
	def SaveAsFileName(self):
		dlg = wx.FileDialog(self, message = "Save as...", 
				defaultDir  = os.path.expanduser('~'), 
				defaultFile = self.defaultFile, 
				wildcard = self.wildcard, style = wx.SAVE | wx.OVERWRITE_PROMPT)

		if dlg.ShowModal() == wx.ID_OK:
			path = dlg.GetPath()
				
		dlg.Destroy()
		
		try:
			return path
		except UnboundLocalError:
			return None
		 
	def WriteFile(self, path, content):
		f = codecs.open(path, 'w', 'utf-8')
		f.write(content)#.encode(self.encoding)
		f.close()
		
	
	
def editInterval(path, tier, interval, txt):
	"Widget to edit a TextGrid interval"
	value = ''
	dlg = wx.TextEntryDialog(
			None, u"Modify interval %d in tier %d :" % (interval, tier),
			u"file " + os.path.basename(path))

	dlg.SetValue(txt)

	if dlg.ShowModal() == wx.ID_OK:
		value = dlg.GetValue()
	dlg.Destroy()

	return value
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
