# -*- coding: utf-8 -*-

import wx
import sys
from messages import *

def escape(s):
	s = s.replace('<', '&lt;')
	s = s.replace('>', '&gt;')
	s = s.replace(' ', '&nbsp;')	
	return s	

class ResultListBox(wx.HtmlListBox):
	"Display results in HTML."
	def __init__(self, tab, pos = wx.DefaultPosition, size = wx.DefaultSize):
		wx.HtmlListBox.__init__(self, tab['win'], pos = pos, size = size)
		self.results = [ ] # this needs to be updated before OnGetItem() is called.
		self.SetHtmlFont()
		
	def __len__(self):
		return len(self.results)
		
	def SetHtmlFont(self):
		if sys.platform.startswith('win'):
                        self.fontFace = 'Courier'
			self.fontSize = 12                     
		elif sys.platform == 'darwin': 
			self.fontFace = 'Monaco'
			self.fontSize = 9 #FIXME: the size is not changed
			
		else:
			self.fontFace = 'Monospace'
			self.fontSize = 12
				

			
	def OnGetItem(self, i):
		if len(self.results) == 0:
			return -1
		item = self.results[i]
		s = escape(item.left) + '<font color = "red"><strong>%s</strong></font>' % item.match + escape(item.right)
		s = '<font face = "%s" size = "%dpt">%s</font>' % (self.fontFace, self.fontSize, s)
		return unicode(s)

	def GetItemAsXhtml(self, i):
		# TODO: merge with OnGetItem
		if len(self.results) == 0:
			return -1
		item = self.results[i]
		
		return escape(item.left) + '<strong>%s</strong>' % item.match + escape(item.right)
		
		
	def ToHtml(self):
		lines = [ ]
		
		if len(self) == 0:
			return None
		
		for i in range(len(self)):
			lines.append(self.GetItemAsXhtml(i))
			
		return \
		u"""<?xml version="1.0" encoding="utf-8"?>
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">		
<html>
<head><title>Results</title></head>
<body>
%s
</body>
</html>
		""" % '<br />'.join(lines)

	def ToCsv(self):
		"Save results as CSV file"
		csv = [ "left_context;match;right_context;previous_word;word;next_word;survey;speaker;task;sex;age;job;description" ]
		
		for item in self.results:
			meta = item.task.speaker.meta
			try:
				wd0, wd1, wd2 = item.getCategories()
				if wd0 == None:
					wd0 = ''
				if wd2 == None:
					wd2 = ''
			except: #TODO: something more subtle than that
				wd0 = wd1 = wd2 = ""
			csv.append(";".join([ item.left.replace(';', ','), item.match, item.right.replace(';', ','), wd0, wd1, wd2, meta['Survey'], item.task.speaker.id, 
					item.task.id, meta['Sex'], str(meta['Age']), meta['Job'], meta['Description'].replace('\n', ' ') ])) 
			
		return "\n".join(csv)
			
	
class WordListChooser:
	def __init__(self, tab, browser, pac=False, pos = (300, 0)):
		self.tab = tab
		self.browser = browser
		x, y = pos
		if pac:
			list  = wx.RadioButton(tab['win'], -1, u"Liste voyelles", pos = (x, y), style = wx.RB_GROUP)
			listx = wx.RadioButton(tab['win'], -1, u"Liste consonnes", pos = (x, y+30))
			self.lists = { 'v': list, 'c': listx }
			self.selected = "v"			
		else:
			list  = wx.RadioButton(tab['win'], -1, u"Liste standard", pos = (x, y), style = wx.RB_GROUP)
			listx = wx.RadioButton(tab['win'], -1, u"Liste complémentaire", pos = (x, y+30))
			self.lists = { 'm': list, 'mx': listx }
			self.selected = "m"
		
		for lst in [list, listx]:
			tab['win'].Bind(wx.EVT_RADIOBUTTON, self.OnSelect, lst)
			
	def OnSelect(self, e):
		# register which task is currently selected
		object = e.GetEventObject()
		label = object.GetLabel().split(' ')[-1]
		if label == u"standard":
			self.selected = "m"
		elif label == u"complémentaire":
			self.selected = "mx"
		elif label == u"voyelles":
			self.selected = "v"
		elif label == u"consonnes":
			self.selected = "c"
		else:
			label = None
		
		# update list 
		self.browser.ResetUIWordList(self.tab)
		
	def DisableTask(self, lst):
		self.lists[lst].Disable()
