# -*- coding: utf-8 -*-

import os
import wx
from messages import info
from scripting import *
from plateforme import version
from editors import TextEditor
import strings
import behaviours


class PFCMenu:
	def __init__(self, parent, prefsdir):
		menuBar = wx.MenuBar()
		self.parent = parent
		# file menu
		fileMenu = wx.Menu()
		
		openMenuItem = fileMenu.Append(wx.ID_OPEN, u'&Ouvrir le corpus...\tctrl-O')
		serialMenuItem = fileMenu.Append(wx.ID_SAVE, u'Enregistrer l\'état du corpus')
		fileMenu.AppendSeparator()	
		
		zipMenuItem = fileMenu.Append(-1, u"Archiver un corpus...")	
		
##		saveAsMenuItem = fileMenu.Append(-1, '&Enregistrer...\tctrl-s')
##		fileMenu.AppendSeparator()	
		
		scriptsMenu = self.SetScriptsMenu()		

		importMenu = wx.Menu()
		scriptImportMenuItem = importMenu.Append(-1, 'Script...')
		pluginImportMenuItem = importMenu.Append(-1, 'Greffon...')		
		metaImportMenuItem = importMenu.Append(-1, u'Métadonnées...')

		fileMenu.AppendMenu(-1,'Import', importMenu)
		fileMenu.AppendSeparator()			
		
		quitMenuItem = fileMenu.Append(wx.ID_EXIT, '&Quitter\tctrl-Q')

		# help menu		
		helpMenu = wx.Menu()
		updateMenuItem = helpMenu.Append(-1, u'Vérifier les mises à jour')
		helpMenu.AppendSeparator()
		helpMenuItem = helpMenu.Append(wx.ID_HELP, u'Documentation\tctrl-H')
		licenseMenuItem = helpMenu.Append(-1, u'Licence')
		aboutMenuItem = helpMenu.Append(wx.ID_ABOUT, u'A propos')


        # edit menu
		if sys.platform.startswith('darwin'):
			prefsMenuItem = helpMenu.Append(wx.ID_PREFERENCES, u'Préférences')
		else:		
			editMenu = wx.Menu()
	#		editMenu.AppendSeparator()
			prefsMenuItem = editMenu.Append(wx.ID_PREFERENCES, u'Préférences')


		menuBar.Append(fileMenu, 'Fichier')
		if not sys.platform.startswith('darwin'):
			menuBar.Append(editMenu, 'Edition')
		menuBar.Append(scriptsMenu, 'Scripts')
		menuBar.Append(helpMenu, '&Aide')

		updateFunc = lambda e: behaviours.CheckUpdate(parent, False)
		scriptFunc = lambda e: importScript(prefsdir)
		pluginFunc = lambda e: importPlugin(prefsdir)
		metaFunc   = lambda e: behaviours.OnImportMetadata(parent)
		serialFunc = lambda e: behaviours.OnSerializeCorpus(parent)
		
		# bindings			
		parent.Bind(wx.EVT_MENU, parent.OnReopen, openMenuItem)
		parent.Bind(wx.EVT_MENU, serialFunc, serialMenuItem)
		parent.Bind(wx.EVT_MENU, behaviours.ZipFolder, zipMenuItem)
		parent.Bind(wx.EVT_MENU, parent.OnQuit, quitMenuItem)
		parent.Bind(wx.EVT_MENU, self.OnAbout, aboutMenuItem)	
		parent.Bind(wx.EVT_MENU, self.OnLicense, licenseMenuItem)
		parent.Bind(wx.EVT_MENU, updateFunc, updateMenuItem)
		parent.Bind(wx.EVT_MENU, parent.OnPrefs, prefsMenuItem)	
		parent.Bind(wx.EVT_MENU, scriptFunc, scriptImportMenuItem)	
		parent.Bind(wx.EVT_MENU, pluginFunc, pluginImportMenuItem)
		parent.Bind(wx.EVT_MENU, metaFunc, metaImportMenuItem)
		parent.Bind(wx.EVT_MENU, behaviours.OnDocumentation, helpMenuItem)
				
		parent.SetMenuBar(menuBar)
		
	def SetScriptsMenu(self):
		"""
		The Scripts Menu is generated dynamically: it imports Praat scripts tagged as PFC 
		scripts (+ optional python code).
		It looks in the app directory and in the user's directory for a dir named 'scripts' 
		and imports properly tagged scripts (see scripting.py).
		"""
		
		paths = getScripts(self.GetScriptsDirs())
		self.scripts = getIncludeScripts(paths)
			
		menu = wx.Menu()
		
		paths = self.scripts.keys()
		paths.sort()
		for path in paths:
			script = self.scripts[path]
			if 'access' in script:
				label = script['label'] + '\tctrl-%s' % script['access']
			else:
				label = script['label']
				
			item = menu.Append(-1, label)
			#FIXME: where does this appear?
			if 'short_desc' in script:
				item.SetHelp(script['short_desc'])
			
			self.BindScriptMenuItem(path, item)

		#parent.Bind(wx.EVT_MENU, parent.OnGen, genMenuItem)	
				
		return menu
	
	def GetScriptsDirs(self):
		dirs = [ ]
		# user prefs
		dirs.append(os.path.join(self.parent.prefsdir, 'scripts'))
		# assume we're in the application folder
		dirs.append(os.path.join(os.path.abspath(os.curdir), 'scripts'))
		
		return dirs
	
	def BindScriptMenuItem(self, path, menuItem):
		func = lambda e: self.ScriptHandler(path)
		
		self.parent.Bind(wx.EVT_MENU, func, menuItem)
		
	def ScriptHandler(self, path):
		script = self.scripts[path]
		praat  = self.parent.praat['pathcmd']

		ScriptForm(self.parent, path, script, praat)
		
	def OnLicense(self, e):
		TextEditor(self.parent, 'Licence', strings.license)
		
	def OnAbout(self, e):
		title = 'Plateforme PFC v ' + version
		msg = u'''
Outil développé par Julien Eychenne.
© Julien Eychenne et Projet PFC, 2007-2009 
<http://www.projet-pfc.net>

Ce programme est écrit en Python et utilise la bibliothèque graphique wxPython.

Remarques, suggestions, bugs : <jeychenne@gmail.com>


Icônes (sauf Praat) : © Everaldo.com
<http://www.everaldo.com>

Praat et son icône: © Paul Boersma & David Weenink
<http://www.praat.org>
		'''
		#wx.lib.dialogs.ScrolledMessageDialog(None, msg, title)
		info(msg, title)


		
		
