# -*- coding: utf-8 -*-

import wx
import sys

class MetaSearchBox:
	"Search boxes speaker file metadata."
	def __init__(self, browser, tab, pos, pos2):
		win = tab['win']
		self.browser = browser

		self.data = { }
		self.choices = { }
		self.extra = { } # store survey and task metadata
		
		processList = lambda e: browser.OnFilterSpeakers(tab)
		processQuery = lambda e: browser.RunQuery(tab)		
		
		searchTip = wx.ToolTip(u"Pour séparer des valeurs, utilisez le point-virgule (ex : Etudiant;vigneron)")
		
		### Process speakers
		x, y = pos #(250, 400)
		sy = 20 # y for size
		if sys.platform.startswith('linux'):
			sy = 25
		x2, x3 = (x+100, x+260)
		
		box = wx.StaticBox(win, -1, u'Métadonnées locuteurs :', pos=(x,y), size=(380, 160)) 
		x += 20; y += 30
		wx.StaticText(win, -1, 'Sexe', pos=(x,y))
		self.choices['Sex'] = wx.Choice(win, -1, pos=(x2, y), size=(150,sy), 
			choices = [u'indifférent', 'Femme', 'Homme'])	
		
		y += 30
		self.data['Age'] =  wx.TextCtrl(win, -1, "", pos=(x, y), size=(70,sy), 
			style=wx.TE_PROCESS_ENTER)
		self.choices['Age'] = wx.Choice(win, -1, pos=(x2, y), size=(50,sy),
				choices = ['<', '=', '>'])
		wx.StaticText(win, -1, 'Age', pos=(x2+60,y))				
		self.choices['Age2'] = wx.Choice(win, -1, pos=(x3-60, y),  size=(50,sy),
				choices = ['<', '=', '>'])
		self.data['Age2'] =  wx.TextCtrl(win, -1, "", pos=(x3, y), style=wx.TE_PROCESS_ENTER)


		self.choices['Age2'].SetSelection(0)
		
		y += 30
		wx.StaticText(win, -1, 'Occupation', pos=(x,y))
		self.choices['Job'] = wx.Choice(win, -1, pos=(x2, y), size=(150,sy),
				choices = ["est", "n'est pas"])
		self.data['Job'] =  wx.TextCtrl(win, -1, "", pos=(x3, y), style=wx.TE_PROCESS_ENTER)
		self.choices['Job'].SetSelection(0)
		self.data['Job'].SetToolTip(searchTip)
		
		y += 30
		wx.StaticText(win, -1, 'Description', pos=(x,y))
		self.choices['Description'] = wx.Choice(win, -1, pos=(x2, y), size=(150,sy),
				choices = ["contient", "ne contient pas"])
		self.data['Description'] =  wx.TextCtrl(win, -1, "", pos=(x3, y), style=wx.TE_PROCESS_ENTER)
		self.choices['Description'].SetSelection(0)
		self.data['Description'].SetToolTip(searchTip)

		if tab is browser.list:
			for key in ['Age', 'Age2', 'Job', 'Description']:
				browser.Bind(wx.EVT_TEXT_ENTER, processList, self.data[key])				
		elif tab is browser.query:
			for key in ['Age', 'Age2', 'Job', 'Description']:
				browser.Bind(wx.EVT_TEXT_ENTER, processQuery, self.data[key])
	
		### Process files and surveys
		x, y = pos2
		
		box2 = wx.StaticBox(win, -1, u'Autres métadonnées :', pos=(x,y), size=(360, 160))

		x += 20; y += 30
		z = 150 # width of choice
		x2 = x + z + 20
		for key, label in [('SurveyDesc', u"l'enquête"), ('TaskDesc', u'la tâche')]:
			wx.StaticText(win, -1, u'Description de %s :' % label, pos=(x,y))
			y += 30
			self.choices[key] = wx.Choice(win, -1, pos=(x, y), size=(z,sy),
				choices = ['contient', 'ne contient pas'])
			self.choices[key].SetSelection(0)
			self.extra[key] = wx.TextCtrl(win, -1, "", pos=(x2, y), size=(150, sy), style=wx.TE_PROCESS_ENTER)
			self.extra[key].SetToolTip(searchTip)
			if tab is browser.list:
				browser.Bind(wx.EVT_TEXT_ENTER, processList, self.extra[key])				
			elif tab is browser.query:
				browser.Bind(wx.EVT_TEXT_ENTER, processQuery, self.extra[key])
			y += 30
			
			
	def sendSpeakerMetaSearch(self):
		meta = self.browser.metadata
		
		for choice in self.choices.keys():
			meta['choices'][choice] = self.choices[choice].GetSelection()

		for datum in self.data.keys():
			meta['values'][datum] = self.data[datum].GetValue()
			
		for extra in self.extra.keys():
			meta['extra'][extra] = self.extra[extra].GetValue()
			
		### make values readable by the browser		
		if meta['choices']['Sex'] == -1 or meta['choices']['Sex'] == 0:
			meta['choices']['Sex'] = ''
		elif meta['choices']['Sex'] == 1:
			meta['choices']['Sex'] = 'Femme'
		elif meta['choices']['Sex'] == 2:
			meta['choices']['Sex'] = 'Homme'
		
		for age in ('Age', 'Age2'):
			if meta['choices'][age] == 0:
				meta['choices'][age] = '<'
			elif meta['choices'][age] == 1:
				meta['choices'][age] = '=='
			elif meta['choices'][age] == 2:
				meta['choices'][age] = '>'
		
		# 'is'/'contains' is 0 so we convert it to a boolean value
		for key in ['Job', 'Description', 'SurveyDesc', 'TaskDesc']:
			meta['choices'][key] = not bool(meta['choices'][key])
