# -*- coding: utf-8 -*-

if __name__ == '__main__':
    import sys
    sys.path.append('..')
import re
from core.regex import Pattern 


"""
Hide and unhide PFC-specific information from a tier.
"""

global MISCAT 
MISCAT = {
    'd[': "d'[PRP]",
    'c[':"c'[PRO:DEM]",
    'j[': "j'[PRO:PER]",
    'qu[': "qu'[KON]",
    'l[': "l'[PRO:PER]", # could be DET but doesn't really matter for PFC
}

def preprocess(tier):
    "Comment PFC-specific information in a tier (received as a list) and return a new list."
    newtier = _hideConventions(tier)
    newtier = _splitIntervals(newtier)
    #_hideLiaison(newtier)
    #_hideSchwa(newtier)
    return newtier
    

def postprocess(tier):
    "Unhide PFC-specific information in a tier (received as a list) and return a new list."
    newtier = _unhideConventions(tier)
    #newtier = _unhideLiaison(newtier)
    #newtier = _unhideSchwa(newtier)
    newtier = _joinIntervals(newtier)
    return newtier


def _hideConventions(tier):
    "Hide PFC conventions."
    newtier = [ ]
    openbr  = Pattern("<([A-Za-z0-9]{1,4})\s?:") 
    spkr    = Pattern("^(.+?)\s?:")
    sampa   = Pattern("\(?\[(.+?)\]\)?") # '[sampa]' or '([sampa])'
    trunc   = re.compile("(\s|')(\w+?(?:-\w+?)?)/([\s,'\.?])", re.UNICODE) # truncated words (can be "beau-f/")
    heswd   = re.compile("/-(\w+?)([\s,\.?])", re.UNICODE) # words containing a hesitation
    comment = re.compile("(\(.+?\))")  
      
    for interval in tier:
        interval = interval.replace ('>', '<CLOSETURN>') # must be substituted first
        interval = openbr.substitute(r'<OPENTURN-\1>', interval)
        interval = spkr.substitute(r'<SPEAKER-\1>', interval)
        interval = sampa.substitute(r'<SAMPA-\1>', interval)
        interval = heswd.sub(r"\1\2 <HES-\1>", interval)
        interval = trunc.sub(r"\1<TRUNC-\2>\3", interval)
        #FIXME: applied twice because some matches overlap (e.g. "mo/ m/").
        interval = trunc.sub(r"\1<TRUNC-\2>\3", interval)
        
        # "(bruit de fond)" becomes "<COMMENT-bruit_de_fond>"
        for cmt in comment.findall(interval):
            newcomment = '<COMMENT-%s>' % cmt[1:-1].replace(' ', '_')
            interval = interval.replace(cmt, newcomment)
        
        newtier.append(interval)
        
    return newtier

def _unhideConventions(tier):
    "Receives list of tag lists and returns a new list"
    global MISCAT
    newtier = [ ]
    for int in range(len(tier)):
        newtier.append([ ])
        
        for wd in range(len(tier[int])):       
            tag = tier[int][wd]  
            app = True # append tag
            
            if tag.startswith('<SPEAKER'):
                # catch exception if conventions are not respected (e.g. "(bruit) E: blabla")
                try:
                    junk, spkr = tag[1:-1].split('-')
                    tag = spkr + ':'
                except ValueError:
                    print 'pb de balisage :', str(tag) 
            # this must go in spokenfrench   
            elif tag.startswith('<DISC'):
                junk, word = tag[1:-1].split('-')
                tag = word + '[DYS]' # dysfluence
            elif tag == '<CLOSETURN>':
                tag = '>'
            elif tag.startswith('<OPENTURN'):
                try:
                    junk, spkr = tag[1:-1].split('-')
                    tag = '<%s:' % spkr
                except ValueError:
                    print 'pb de balisage :', str(tag)
            elif tag.startswith('<COMMENT'): 
                try:
                    junk, word = tag[1:-1].split('-')
                except:
                    print "Too many values to unpack in comment"
                    print tag
                tag = '(%s)' % word.replace('_', ' ')           
            elif tag.startswith('<TRUNC'): 
                try:
                    junk, word = tag[1:-1].split('-')
                except:
                    junk, wd1, wd2 = tag[1:-1].split('-')
                    word = wd1 + "-" + wd2
                tag = word + "/[TRN]"
            elif tag.startswith('<HES'):
                junk, word = tag[1:-1].split('-') 
                # handle cases like a/-griculteur.
                hes = newtier[-1][-1]
                idx = -1
                if hes.startswith(".") or hes.startswith(","):
                    idx = -2
                    hes = newtier[-1][idx] 
                # find cut in tagged word
                pos = hes.find('[') - len(word) 
                newtier[-1][idx] =  hes[:pos] + '/-' + hes[pos:] 
                app = False
                
            elif tag.startswith('<SAMPA'):
                try:
                    junk, word = tag[1:-1].split('-')
                except:
                    print tag
                tag = "[%s][SAMPA]" % word
            
            #TODO: handle mistagging in a better way 
            elif tag == "'":
                try:
                    for idx in [2,3]:
                        if newtier[-1][-1][:idx] in MISCAT.keys():
                            newtier[-1][-1] = MISCAT[ newtier[-1][-1][:idx] ]
                            app = False        
                except IndexError:
                    pass
                
            if app:        
                newtier[int].append(tag)
            
    return newtier

#------------------------------------------------------------------------------------------
def _hideSchwa(tier):
    schwa = re.compile('(\d\d\d\d)')
    _hideCodings(tier, schwa, 'SCHWA')
    
        
def _hideLiaison(tier):
    'Compile the string coding into a liaison pattern.'
    ptn = '([0-4][0-4][bcdfgjklmnpqrstvwxzJRSZ]?(?:VN|VO)?[hC]?)' 
    liais = re.compile(ptn)
    _hideCodings(tier, liais, 'LIAISON')

    
def _hideCodings(tier, ptn, flag):
    """Each word is scanned for codings, which are stripped off the word and remembered
    according to their index in the word and position, 
    e.g. 'mainte1312nant' -> 'maintenant <SCHWA-1312-0-5>."""
    # Hiding is operated on the tier itself (no new tier created) to avoid unnecessary computations.
    for int in range(len(tier)):        
        for wd in range(len(tier[int])):
            word = tier[int][wd]
            tail = ''
            
            # set an index for each coding (relevant for schwa only)
            index = 0
            while True:
                match = ptn.search(word)
                if not match:
                    break
                
                coding = match.group(0)
                pos = match.start(0)
                # at this stage, sequences like "s'occupe" are considered one "word",
                # but they will be split by the TreeTagger (except "grand'père", etc.)
                # The position is adjusted accordingly.
                apostr = word.find("'") + 1
                if apostr > 0 and not word.startswith("grand'") \
                    and word.lower() != "d'accord":
                    newpos = pos - apostr            
                else:
                    newpos = pos
                size = len(coding)
                word = word[:pos] + word[pos+size:]
                tail += '<%s-%s-%d-%d>' % (flag, coding, index, newpos)
                
                index += 1
            
            # append the coding string to the bare word
            if tail:
                tier[int][wd] = word + ' ' + tail

#-----------------------------------------------------------------------------------------
def _unhideSchwa(tier):
    return _unhideCodings(tier, 'SCHWA') 

def _unhideLiaison(tier):
    return _unhideCodings(tier, 'LIAISON') 

def _unhideCodings(tier, flag):
    newtier = [ ]
    
    for intv in range(len(tier)):
        newtier.append([ ])
        for wd in range(len(tier[intv])):
            tag = tier[intv][wd]
            # ["revenu", "<SCHWA-1212-0-X>", "<SCHWA-1312-1-X>"], If the tag doesn't start 
            # with '<SCHWA', it means there are no more schwa codings for this word.
            if tag.startswith('<' + flag):
                try:
                    junk, coding, index, pos = tag[1:-1].split('-')
                except:
                    print tier[intv][wd-2]
                    print tier[intv][wd-1]
                    print tier[intv][wd]
                    print tier[intv][wd+1]
                    print tier[intv][wd+2]                    
                    print
                # TODO: adapt to deal with liaison
                # 4 characters is the size of the schwa coding
                pos = int(pos) + (int(index) * 4)
                # append on the previous word
                newtier[-1][-1] = newtier[-1][-1][:pos] + coding  \
                + newtier[-1][-1][pos:]  
            else:
                newtier[intv].append(tag)
    
    return newtier


#-----------------------------------------------------------------------------------------
def _splitIntervals(tier):
    "Every interval is returned as a list instead of a string."
    newtier = [ ]
    
    for interval in tier:
        interval = interval.replace('  ', ' ') # no multiple spaces
        newint = [ ]
        
        for wd in interval.split():
            if ispunct(wd[-1]):
                newint.append(wd[:-1])
                newint.append(wd[-1])
            else:
                newint.append(wd)               
        newtier.append(newint)
        
    return newtier

def _joinIntervals(tier):
    "Every interval is returned as string instead of a list."
    newtier = [ ]
    
    for words in tier:
        interval = ' '.join(words)
        interval = interval.replace(' ,', ',').replace(' .', '.')
        newtier.append(interval)
        
    return newtier

def ispunct(c):
    return c == ',' or c == '.' or c == '?'

#-----------------------------------------------------------------------------------------
def _intervalsToString(tier):
    "Create a string from a list of intervals. Interval boundaries are marked as '<I>'."
    for i in range(len(tier)):
        tier[i] = ' '.join(tier[i])
        
    return '<I>'.join(tier)

def _tagsToIntervals(tags):
    "Recreate a list of intervals from the treetagger's output"
    tier = [[]]
    
    for tag in tags:
        if tag == '<I>':
            tier.append([ ])
        else:
            tier[-1].append(tag)
            
    return tier
    
def tag():
    "main tagging function from the command line"
    pass

if __name__ == '__main__':
    import wx
    import os
    app = wx.PySimpleApp()
    from treetagger.spokenfrench import SpokenFrenchTagger as Tagger
    from pfc import Corpus
    
    corpus = Corpus('~/PFC/CorpusTag', gui = False)
 
    # test de l'integration du tagging
    test = False
    
    if test:
        tgd = corpus['11a']['al1']['t'].textgrid
        tgd.splitCodingTiers()
        
        print tgd.splitTags[2][2]
        print tgd.splitSchwa[2][2]
        print tgd.splitLiaison[2][2]
    
    else:
        tagger = Tagger('/usr/local/treetagger')

        srv = '11a'
        spkr = 'tg1'

        datadir = os.path.join(corpus.path, srv, srv+spkr, 'data') 
        tagdir = os.path.join(datadir, 'tagging')
        if not os.path.exists(datadir):
            os.mkdir(datadir)
        if not os.path.exists(tagdir):
            os.mkdir(tagdir)
            
        
        for tsk in ['g', 'l', 't']:
            tierobj = corpus[srv][spkr][tsk]['transcription'] #transcription
            tier = tierobj.getText()
        
            tier = preprocess(tier) 
            tier = tagger.tag(tier)
            tier = postprocess(tier)
            
            tierobj.setText(tier)
            outfile = os.path.join(tagdir, '%s%s%s-tag.IntervalTier' % (srv, spkr, tsk))
            tierobj.write(outfile)

    app.MainLoop() 
