# -*- coding: utf-8 -*-

"""
This module contains a generic-ish tagger for spoken French.
"""
import re
from treetaggerwrapper import TreeTagger

global disc
disc = \
[
    ('euh|heu', '<DISC-euh>'),
    ('enfin', '<DISC-enfin>'),
    (u'voilàﾠ', u'<DISC-voilà>'), 
    ('en fait', '<DISC-en_fait>'),
    ('hum', '<DISC-hum>'),
    ('ben', '<DISC-ben>'),
    ('hein', '<DISC-hein>'), 
    ('eh', '<DISC-eh>'),
    (u'hé', u'<DISC-hé>'),
    (u'bé|bè', u'<DISC-bè>'),
    ('pfff?', '<DISC-pff>'),
]


class SpokenFrenchTagger:
    def __init__(self, path):
        self.tagger = TreeTagger(TAGLANG = 'fr', TAGDIR = path)
        
    def tag(self, lines, punctuation = False):
        # BROKEN
        #lines = self.pretag(lines)
        ##text = self._hideRepetitions(text)
        # treetagger uses latin-1        
        tags = map(lambda s: unicode(s, 'latin-1'), 
            self.tagger.TagText(_linesToString(lines).encode('latin-1')))
        
        text = TagText(tags)
    
        return _tagsToLines(text.rearrange(punctuation))

    def pretag(self, lines):  
        newlines = [ ]  
       
        for line in lines:
            for old, new in disc:
                ptn = re.compile(u'(\W)(?:%s)(\W)' % old, re.IGNORECASE|re.UNICODE)   
                ptn.sub(r'\1%s\2' % new, line)
            newlines.append(line) 
                
        return newlines
    
    def _hideRepetitions(self, lines):
        newlines = [ ]
        rep1 = re.compile(r"(.+?)(,?\s)\1")
        
        for line in lines:
            # one word repetitions (eg. "il, il est parti")
            line = rep1.sub(r"<REP-\2> \1", line)
            line = line.replace('<REP-, >', '<REP-V>').replace('<REP- >', '<REP-E>').replace('<REP- >', '<REP-E>')
    
            # postprocess "nous nous" and "vous vous".
            line = line.replace('<REP-E> nous ', 'nous nous ').replace('<REP-E> vous ', 'vous vous ')
            newlines.append(line)
        return newlines

class TagText:
    def __init__(self, tags):
        self.tags = tags
    
    def rearrange(self, punctuation):
        "Change 'word\tcat' to 'word[cat]'. Punctuation tagging is optional."       
        newtags = [ ]
        
        for string in self.tags:
            tag = string.split('\t')
            if len(tag) > 1:
                if tag[1] == 'PUN' and not punctuation:
                    newtags.append(tag[0])
                else:
                    newtags.append("%s[%s]" % (tag[0], tag[1]))
            else:
                newtags.append(tag[0])      
                
        return newtags

       
def _linesToString(tier):
    "Create a string from a list of lines. Boundaries are marked as '<LN>'."
    for i in range(len(tier)):
        tier[i] = ' '.join(tier[i])
        
    return '<LN>'.join(tier)

def _tagsToLines(tags):
    "Recreate a list of lines from the treetagger's output"
    tier = [[]]
    
    for tag in tags:
        if tag == '<LN>':
            tier.append([ ])
        else:
            tier[-1].append(tag)
            
    return tier
        
   